'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.detectAll = exports.detectSDK = exports.detectDevice = exports.detectBrowser = exports.detectApp = exports.detectOS = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _rules = require('./rules');

var detector = function detector(rules, handler) {
  return function (userAgent) {
    for (var i = 0; i < rules.length; i++) {
      var rule = rules[i];
      try {
        var _rule = _slicedToArray(rule, 2),
            name = _rule[0],
            reg = _rule[1];

        var matches = userAgent.match(reg);
        var res = handler && handler(matches, name);
        if (res) {
          return res;
        }
      } catch (err) {
        continue;
      }
    }

    return _rules.defaults;
  };
};

var defaultHandler = function defaultHandler(matches, name) {
  if (matches && matches.length > 1) {
    var groups = matches.groups;
    if (groups) {
      return {
        name: groups.name || _rules.defaults.name,
        version: groups.version || _rules.defaults.version
      };
    } else {
      var version = matches[1];
      if (version.indexOf('_') > -1) {
        version = version.split('_').join('.');
      }
      return {
        name: name,
        version: version
      };
    }
  }
  return null;
};

var mergePatches = function mergePatches(patches, origin) {
  if (patches && patches.length) {
    return patches.concat(origin);
  }
  return origin;
};

var detectOS = exports.detectOS = function detectOS(patches) {
  return detector(mergePatches(patches, _rules.OS), function (matches, name) {
    if (matches && matches.length) {
      var version = matches[1];

      if (name === 'Windows') {
        version = _rules.WIN_OS_NAME[version] || version;
      }
      if (version.indexOf('_') > -1) {
        version = version.split('_').join('.');
      }
      return {
        name: name,
        version: version
      };
    }
    return null;
  });
};

var detectApp = exports.detectApp = function detectApp(patches) {
  return detector(mergePatches(patches, _rules.APP), defaultHandler);
};

var detectBrowser = exports.detectBrowser = function detectBrowser(patches) {
  return detector(mergePatches(patches, _rules.BROWSER), defaultHandler);
};

var detectDevice = exports.detectDevice = function detectDevice(patches) {
  return detector(mergePatches(patches, _rules.DEVICE), function (matches, name) {
    var v = void 0;
    if (matches && matches.length > 0) {
      if (name === 'Samsung') {
        v = matches[3];
      } else if (name === 'iPhone') {
        v = matches[1] && matches[1].replace(/_/g, '.');
      } else if (name === 'vivo') {
        v = matches[2];
      } else {
        v = matches[1];
      }

      return {
        name: name,
        version: v || _rules.defaults.version
      };
    }
    return null;
  });
};

var detectSDK = exports.detectSDK = function detectSDK(patches) {
  return detector(mergePatches(patches, _rules.SDK), defaultHandler);
};

var detectAll = exports.detectAll = function detectAll() {
  var patches = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function () {
    var ua = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    var osPatches = patches.osPatches,
        appPatches = patches.appPatches,
        browserPatches = patches.browserPatches,
        devicePatches = patches.devicePatches,
        sdkPatches = patches.sdkPatches;

    return {
      os: detectOS(osPatches)(ua),
      app: detectApp(appPatches)(ua),
      browser: detectBrowser(browserPatches)(ua),
      device: detectDevice(devicePatches)(ua),
      sdk: detectSDK(sdkPatches)(ua)
    };
  };
};